@@ -154,7 +154,7 @@ read_srclines (Dwarf *dbg,
   int res = -1;
 
   size_t nfilelist = 0;
-  unsigned int ndirlist = 0;
+  size_t ndirlist = 0;
 
   /* If there are a large number of lines, files or dirs don't blow up
      the stack.  Stack allocate some entries, only dynamically malloc
@@ -182,6 +182,17 @@ read_srclines (Dwarf *dbg,
       .discriminator = 0
     };
 
+  /* The dirs normally go on the stack, but if there are too many
+     we alloc them all.  Set up stack storage early, so we can check on
+     error if we need to free them or not.  */
+  struct dirlist
+  {
+    const char *dir;
+    size_t len;
+  };
+  struct dirlist dirstack[MAX_STACK_DIRS];
+  struct dirlist *dirarray = dirstack;
+
   if (unlikely (linep + 4 > lineendp))
     {
     invalid_data:
@@ -299,7 +310,7 @@ read_srclines (Dwarf *dbg,
   };
 
   /* First count the entries.  */
-  unsigned int ndirs = 0;
+  size_t ndirs = 0;
   if (version < 5)
     {
       const unsigned char *dirp = linep;
@@ -345,20 +356,19 @@ read_srclines (Dwarf *dbg,
 
       if (nforms == 0 && ndirs != 0)
 	goto invalid_data;
-    }
 
-  struct dirlist
-  {
-    const char *dir;
-    size_t len;
-  };
-  struct dirlist dirstack[MAX_STACK_DIRS];
-  struct dirlist *dirarray = dirstack;
+      /* Assume there is at least 1 byte needed per form to describe
+	 the directory.  Filters out insanely large ndirs.  */
+      if (nforms != 0 && ndirs > (size_t) (lineendp - linep) / nforms)
+	goto invalid_data;
+    }
 
   /* Arrange the list in array form.  */
   ndirlist = ndirs;
   if (ndirlist >= MAX_STACK_DIRS)
     {
+      if (ndirlist > SIZE_MAX / sizeof (*dirarray))
+	goto no_mem;
       dirarray = (struct dirlist *) malloc (ndirlist * sizeof (*dirarray));
       if (unlikely (dirarray == NULL))
 	{
@@ -556,12 +566,17 @@ read_srclines (Dwarf *dbg,
       if (nforms == 0 && nfiles != 0)
 	goto invalid_data;
 
+      /* Assume there is at least 1 byte needed per form to describe
+	 the file.  Filters out insanely large nfiles.  */
+      if (nforms != 0 && nfiles > (size_t) (lineendp - linep) / nforms)
+	goto invalid_data;
+
       Dwarf_Attribute attr;
       attr.cu = &fake_cu;
       for (unsigned int n = 0; n < nfiles; n++)
 	{
 	  const char *fname = NULL;
-	  Dwarf_Word diridx = -1;
+	  Dwarf_Word diridx = (Dwarf_Word) -1;
 	  for (unsigned char m = 0; m < nforms; m++)
 	    {
 	      if (m == form_path)
@@ -576,7 +591,8 @@ read_srclines (Dwarf *dbg,
 		  attr.code = DW_AT_decl_file; /* Close enough.  */
 		  attr.form = forms[m];
 		  attr.valp = (void *) linep;
-		  dwarf_formudata (&attr, &diridx);
+		  if (dwarf_formudata (&attr, &diridx) != 0)
+		    diridx = (Dwarf_Word) -1;
 		}
 
 	      size_t len = __libdw_form_val_len (&fake_cu, forms[m], linep);
@@ -1049,7 +1065,7 @@ read_srclines (Dwarf *dbg,
       free (state.linelist);
       state.linelist = ll;
     }
-  if (ndirlist >= MAX_STACK_DIRS)
+  if (dirarray != dirstack)
     free (dirarray);
   for (size_t i = MAX_STACK_FILES; i < nfilelist; i++)
     {
