@@ -1,5 +1,5 @@
 /* Test program for dwarf_getscopes.
-   Copyright (C) 2005 Red Hat, Inc.
+   Copyright (C) 2005, 2014 Red Hat, Inc.
    This file is part of elfutils.
 
    This file is free software; you can redistribute it and/or modify
@@ -153,15 +153,16 @@ handle_function (Dwarf_Die *funcdie, void *arg)
 	      lowpc += a->dwbias;
 	      highpc += a->dwbias;
 	      Dwfl_Line *loline = dwfl_getsrc (a->dwfl, lowpc);
-	      Dwfl_Line *hiline = dwfl_getsrc (a->dwfl, highpc);
+	      Dwfl_Line *hiline = dwfl_getsrc (a->dwfl, highpc - 1);
 	      paddr (": ", lowpc, loline);
 	      if (highpc != lowpc)
-		paddr (" .. ", lowpc, hiline == loline ? NULL : hiline);
+		paddr (" .. ", highpc - 1, hiline == loline ? NULL : hiline);
 	    }
 	  puts ("");
 
 	  print_vars (indent + INDENT, die);
 	}
+      free (scopes);
     }
 
   return 0;
