@@ -122,7 +122,8 @@ dwarf_form_string (unsigned int form)
 static void
 print_base_type (Dwarf_Die *base)
 {
-  assert (dwarf_tag (base) == DW_TAG_base_type);
+  if (dwarf_tag (base) != DW_TAG_base_type)
+    error (EXIT_FAILURE, 0, "not a base type");
 
   Dwarf_Attribute encoding;
   Dwarf_Word enctype = 0;
@@ -545,7 +546,8 @@ print_expr (Dwarf_Attribute *attr, Dwarf_Op *expr, Dwarf_Addr addr)
 	// XXX actually lookup DW_TAG_GNU_call_site_parameter
 	printf ("%s[%" PRIx64 "]", opname, dwarf_dieoffset (&param));
 	assert (expr->number == dwarf_cuoffset (&param));
-	assert (dwarf_tag (&param) == DW_TAG_formal_parameter);
+	if (dwarf_tag (&param) != DW_TAG_formal_parameter)
+	  error (EXIT_FAILURE, 0, "Not a formal parameter");
       }
       break;
 
@@ -1120,7 +1122,8 @@ main (int argc, char *argv[])
 	  cfi_debug = dwfl_module_dwarf_cfi (mod, &cfi_debug_bias);
 	  cfi_eh = dwfl_module_eh_cfi (mod, &cfi_eh_bias);
 
-	  assert (cfi_debug_bias == 0); // No bias needed, same file.
+	  // No bias needed, same file.
+	  assert (cfi_debug == NULL || cfi_debug_bias == 0);
 
 	  // We are a bit forgiving for object files.  There might be
 	  // relocations we don't handle that are needed in some
